<?php
// inc_global.php
// Global data structure for WPPS

class wpps {
    public $settings = false;
    public $silentmode = false;
    public $filesdir;
    public $plugins = array();
    public $forceupdate = false;

    public function __construct() {
        $this->filesdir = plugin_dir_path(__FILE__) . 'files';
    }

    public function setSilent() {
        $this->silentmode = true;
    }

    public function addPlugin($name) {
        $this->plugins[0] = $name;
    }

    public function updateAll($maxchanges = 99) {
        if ($this->updateShortcodes($maxchanges)) {
            update_option('wpps_lastupdate', current_time('timestamp', 0));
        }
    }

    private function updateShortcodes($maxchanges) {
        global $wpps;

        require_once('inc_fetchrequest.php');

        $data = getDocumentInfo(); // This could return false or an array.

        $updatecount = 0;
        $newcount = 0;
        $nochangecount = 0;
        $failcount = 0;

        if (is_array($data)) {
            foreach ($data as $document) {
                if ($updatecount + $newcount + $nochangecount + $failcount >= $maxchanges) {
                    break;
                }

                $thisdoc = get_page_by_title($document['id'], OBJECT, 'wpps-shortcodes');

                if (!is_null($thisdoc)) {
                    $version = get_post_meta($thisdoc->ID, 'wpps-version', true);
                    if ($version == $document['version'] . '/' . $wpps->settings['format'] && !$this->forceupdate) {
                        $nochangecount++;
                    } else {
                        $doctext = $this->getDocText($document['id'], $wpps->settings['format']);
                        $dochash = $this->getDocHash($document['id'], $wpps->settings['format']);

                        if (md5($doctext) == $dochash) {
                            update_post_meta($thisdoc->ID, 'wpps-version', $document['version'] . '/' . $wpps->settings['format']);
                            update_post_meta($thisdoc->ID, 'wpps-content', $doctext);
                            do_action('wpps-update-document', $document);
                            $updatecount++;
                        } else {
                            $wpps->raiseError('Data integrity check failed for document ' . $document['id']);
                            $failcount++;
                        }
                    }
                } else {
                    $newdoc = wp_insert_post(array(
                        'post_title' => $document['id'],
                        'post_status' => 'publish',
                        'post_type' => 'wpps-shortcodes'
                    ));

                    $doctext = $this->getDocText($document['id'], $wpps->settings['format']);
                    $dochash = $this->getDocHash($document['id'], $wpps->settings['format']);

                    if (md5($doctext) == $dochash) {
                        add_post_meta($newdoc, 'wpps-version', $document['version'] . '/' . $wpps->settings['format'], true);
                        add_post_meta($newdoc, 'wpps-content', $doctext, true);
                        add_post_meta($newdoc, 'wpps-aktiv', '1', true);
                        add_post_meta($newdoc, 'wpps-status', '0', true);
                        add_post_meta($newdoc, 'wpps-lastpostid', '0', true);
                        do_action('wpps-new-document', $document);
                        $newcount++;
                    } else {
                        $wpps->raiseError('Data integrity check failed for document ' . $document['id']);
                        $failcount++;
                    }
                }
            }
        } else {
            $this->raiseWarning('No valid data returned from getDocumentInfo().');
            return false;
        }

        return $updatecount + $nochangecount + $newcount + $failcount == count($data);
    }


    public function getDocText($did, $format) {
        $data = getDocument($did, $format);

        if ($data !== false) {
            return $format == 'Text' ? normalizeText((string)$data->Document) : (string)$data->Document;
        }

        return false;
    }

    public function getDocHash($did, $format) {
        $data = getDocument($did, $format);

        return $data !== false ? (string)$data->MD5 : false;
    }

    public function getShortcodeText($doc, $plaintext = false, $striptags = false) {
        $thisdoc = get_page_by_title($doc, OBJECT, 'wpps-shortcodes');
        $output = '';

        if (get_post_meta($thisdoc->ID, 'wpps-aktiv', true) == '1') {
            update_post_meta($thisdoc->ID, 'wpps-status', '1');
            update_post_meta($thisdoc->ID, 'wpps-lastpostid', get_the_ID());

            $output = !$plaintext ? get_post_meta($thisdoc->ID, 'wpps-content', true) : get_post_meta($thisdoc->ID, 'wpps-plaintext', true);

            if (!$plaintext && $striptags) {
                $output = preg_replace('#<br\s*/?>#i', "\n", $output);
                $output = str_replace('&nbsp;', "\n", $output);
                $output = strip_tags($output);
            }
        }

        return $output;
    }

    public static function installPlugin() {
        update_option('wpps_updatetype', 'auto');
        update_option('wpps_interval', '24');
        update_option('wpps_apiurl', 'https://www.protectedshops.de/api/');
        update_option('wpps_format', 'Html');
    }

    public function raiseSuccess($msg) {
        if (!$this->silentmode && is_admin()) {
            echo '<div class="updated"><p><strong>' . $msg . '</strong></p></div>';
        }
    }

    public function raiseWarning($msg) {
        if (!$this->silentmode && is_admin()) {
            echo '<div class="update-nag"><p><strong>' . $msg . '</strong></p></div>';
        }
    }

    public function raiseError($msg) {
        if (!$this->silentmode && is_admin()) {
            echo '<div class="error"><p><strong>' . $msg . '</strong></p></div>';
        }
    }
}

function wpps_init() {
    register_post_type('wpps-shortcodes', array(
        'labels' => array(
            'name' => __('Shortcodes'),
            'singular_name' => __('Shortcode')
        ),
        'public' => true,
        'has_archive' => false,
        'rewrite' => array('slug' => 'wpps-shortcodes'),
        'supports' => array('title', 'custom-fields'),
    ));
}

add_action('init', 'wpps_init');
?>
